const ESCAPE_LOOKUP = {
  '&': '&amp;',
  '>': '&gt;',
  '<': '&lt;',
  '"': '&quot;',
  '\'': '&#x27;',
}
const ESCAPE_REGEX = /[&><'']/g

global.Rev = class Revelry {

  static appObject(name, object) {
    return Revelry.registerObject(`App.${name}`, object)
  }

  static registerObject(name, object) {
    const parts = name.split('.')
    let container = Revelry

    _(parts).each((part, i) => {

      if(i === parts.length - 1) {
        if(container[part] !== null && typeof container[part] !== 'undefined' && name !== 'App.Components.Main') {
          console.warn(`Object previously defined: ${name}`)
        }
        container[part] = object
      } else {
        container[part] = container[part] || {}
        container = container[part]
      }
    })

    return object
  }

  static appComponent(name, object) {
    name = `App.Components.${name}`
    return Revelry.registerObject(name, Revelry.createReactClass(name, object))
  }

  static registerComponent(name, object) {
    name = `Components.${name}`
    return Revelry.registerObject(name, Revelry.createReactClass(`Rev.${name}`, object))
  }

  static createReactClass(displayName, object) {
    object.displayName = displayName

    // Initialize mixins to an empty array if it doesn't exist yet
    if(!object.mixins) {
      object.mixins = []
    } else if(!(object.mixins instanceof Array)) {
      throw new Error(`${displayName}'s mixins property must be an array.`)
    }

    // Check for inclusion of missing mixins
    object.mixins.forEach((mixin, index) => {
      if(mixin === null || typeof mixin === 'undefined') {
        console.warn(`${displayName} attempted to include a missing mixin at index ${index}.`)
      }
    })

    // Add Revelry Core mixin
    object.mixins.unshift(Revelry.Mixins.Core)

    if(typeof object === 'function') { // ES6 or Coffeescript class, OK as-is
      return object
    }
    // Old style react class - pass to createClass
    return React.createClass(object)
  }

  static appMixin(name, object) {
    if(object === null || typeof object === 'undefined') {
      throw new Error(`Missing mixin \`${name}\``)
    }
    return Revelry.registerObject(`App.Mixins.${name}`, object)
  }

  static registerMixin(name, object) {
    if(object === null || typeof object === 'undefined') {
      throw new Error(`Missing mixin \`${name}\``)
    }
    return Revelry.registerObject(`Mixins.${name}`, object)
  }

  static registerExample(name, object) {
    name = `Examples.${name}`
    return Revelry.registerObject(name, Revelry.createReactClass(`Rev.${name}`, object))
  }

  static appExample(name, object) {
    name = `App.Examples.${name}`
    return Revelry.registerObject(name, Revelry.createReactClass(name, object))
  }

  static registerModel(name, object) {
    return Revelry.registerObject(`Models.${name}`, object)
  }

  static appModel(name, object) {
    return Revelry.registerObject(`App.Models.${name}`, object)
  }

  static pullInto(context) {
    console.warn('`Rev.pullInto` is deprecated.')
    return (...names) => {
      names.forEach((name) => context[name] = Rev.Components[name])
    }
  }

  static _escaper(match) {
    return ESCAPE_LOOKUP[match]
  }

  static escapeTextForBrowser(text) {
    return (`${text}`).replace(ESCAPE_REGEX, Revelry._escaper)
  }

  static _resetUniqueId() {
    let idCounter = 0

    _.uniqueId = function(prefix) {
      const id = `${++idCounter}`

      return prefix ? prefix + id : id
    }
  }

  static hasView(path) {
    return Revelry.Components.Main.hasView(path)
  }

  static viewToString(path, options) {
    Revelry._resetUniqueId()

    const {Main} = Revelry.App.Components

    const props = {
      path,
      options: JSON.parse(options),
    }

    const json = Revelry.escapeTextForBrowser(JSON.stringify(props))

    const tag = (inner) => `<div data-react-class='App.Components.Main' data-react-props='${json}'>${inner}</div>`

    const component = React.createElement(Main, props)

    // 2 checks: One is for standard Rails apps and one for jsonapi-resources.
    if(props.options.disable_prerender || (props.options.meta && props.options.meta['disable-prerender'])) {
      return tag
    }

    try {
      if(React.renderComponentToString !== null && typeof React.renderComponentToString !== 'undefined') { // < 0.13
        return tag(React.renderComponentToString(component))
      } else { // >= 0.13
        return tag(React.renderToString(component))
      }
    } catch(e) {
      return `<pre><code>${path}</code></pre>` +
             `<pre><code>${Revelry.escapeTextForBrowser(e.stack)}</code></pre>` +
             tag('') +
             `<pre><code>${json}</code></pre>`
    }
  }

  static initExecjsRails() {
    global.execjs_rails_has_view = Revelry.hasView.bind(Revelry)
    global.execjs_rails_handler = Revelry.viewToString.bind(Revelry)
  }
}

global.App = Rev.App = {
  Mixins: {},
  Components: {},
  Examples: {},
}
