require "action_controller"
require "action_controller/metal/renderers"
require "execjs"
require "execjs-rails"
require "jquery-rails"
require "react-rails"
require "react/source"
require "backbone-rails"
require "underscore-rails"
require "underscore-string-rails"
require "sass"
require "sass-rails"
require "foundation-rails"
require "sprockets/es6"

module Revelry
  module Core
    class Engine < ::Rails::Engine
      # Configure an asset that depends on application files, e.g. the router
      # which requires the route table to be established before being processed
      # We do this by programmatically adding and initializer that makes the
      # asset depend on the application file
      #   name is the name to use for the initializer
      #   asset is the asset path of the asset to configure
      #   dependency is a callable which returns the application file to depend
      #     on. It is a callable so that we can defer execution until
      #     initialization.
      def self.js_initializer(name, asset, dependency)
        initializer name, after: "sprockets.environment" do
          if Rails.application.assets.respond_to?(:register_preprocessor)
            Rails.application.assets.register_preprocessor 'application/javascript', :'revelry.dependent_on_routes' do |ctx,data|
              ctx.depend_on(dependency.call) if ctx.logical_path == asset
              data
            end
          end
        end
      end

      js_initializer 'revelry.dependent_on_routes',
                     'revelry/router/router',
                     -> { Rails.root.join('config','routes.rb') }

      js_initializer 'revelry.set_js_env',
                      'revelry/env',
                      -> { Rails.root.join('config','application.rb') }

      # Enable controllers to respond to .props format, which returns the same
      # props that execjs uses for serverside render (as a JSON object)
      initializer "revelry.setup_props_renderer", :group => :all do |app|
        Mime::Type.register "text/reactprops", :props

        ActionController.add_renderer :props do |foo, opts|
          self.response_body = render_to_string template: Revelry::Core::PropsTemplate.new('', 'path', ActionView::Template.registered_template_handler(:erb), format: Mime[:erb], virtual_path: 'path')
        end
      end

      initializer 'revelry.set_react_version', before: 'react_rails.setup_vendor' do |app|
        app.config.react.addons = true
        app.config.react.variant = (::Rails.env.production? ? :production : :development)
      end
    end
  end
end
